package com.sun.xml.internal.ws.server;

import javax.xml.ws.Holder;
import java.net.MalformedURLException;
import javax.xml.ws.WebServiceException;
import java.net.URL;
import com.sun.xml.internal.stream.buffer.XMLStreamBufferResult;
import com.sun.xml.internal.stream.buffer.MutableXMLStreamBuffer;
import javax.xml.transform.Result;
import java.util.Iterator;
import com.sun.xml.internal.ws.api.server.SDDocument;
import java.util.HashMap;
import java.util.ArrayList;
import com.sun.istack.internal.NotNull;
import javax.xml.namespace.QName;
import java.util.Map;
import com.sun.xml.internal.ws.api.server.SDDocumentSource;
import java.util.List;
import com.oracle.webservices.internal.api.databinding.WSDLResolver;

final class WSDLGenResolver implements WSDLResolver
{
    private final List<SDDocumentImpl> docs;
    private final List<SDDocumentSource> newDocs;
    private SDDocumentSource concreteWsdlSource;
    private SDDocumentImpl abstractWsdl;
    private SDDocumentImpl concreteWsdl;
    private final Map<String, List<SDDocumentImpl>> nsMapping;
    private final QName serviceName;
    private final QName portTypeName;
    
    public WSDLGenResolver(@NotNull List<SDDocumentImpl> docs, QName serviceName, QName portTypeName) {
        this.newDocs = new ArrayList();
        this.nsMapping = new HashMap();
        this.docs = docs;
        this.serviceName = serviceName;
        this.portTypeName = portTypeName;
        for (SDDocumentImpl doc : docs) {
            if (doc.isWSDL()) {
                SDDocument.WSDL wsdl = (SDDocument.WSDL)doc;
                if (wsdl.hasPortType()) {
                    this.abstractWsdl = doc;
                }
            }
            if (doc.isSchema()) {
                SDDocument.Schema schema = (SDDocument.Schema)doc;
                List<SDDocumentImpl> sysIds = (List)this.nsMapping.get(schema.getTargetNamespace());
                if (sysIds == null) {
                    sysIds = new ArrayList();
                    this.nsMapping.put(schema.getTargetNamespace(), sysIds);
                }
                sysIds.add(doc);
            }
            else {
                continue;
            }
        }
    }
    
    public Result getWSDL(String filename) {
        URL url = this.createURL(filename);
        MutableXMLStreamBuffer xsb = new MutableXMLStreamBuffer();
        xsb.setSystemId(url.toExternalForm());
        this.concreteWsdlSource = SDDocumentSource.create(url, xsb);
        this.newDocs.add(this.concreteWsdlSource);
        XMLStreamBufferResult r = new XMLStreamBufferResult(xsb);
        r.setSystemId(filename);
        return r;
    }
    
    private URL createURL(String filename) {
        try {
            return new URL("file:///" + filename);
        }
        catch (MalformedURLException e) {
            throw new WebServiceException(e);
        }
    }
    
    public Result getAbstractWSDL(Holder<String> filename) {
        if (this.abstractWsdl != null) {
            filename.value = this.abstractWsdl.getURL().toString();
            return null;
        }
        URL url = this.createURL((String)filename.value);
        MutableXMLStreamBuffer xsb = new MutableXMLStreamBuffer();
        xsb.setSystemId(url.toExternalForm());
        SDDocumentSource abstractWsdlSource = SDDocumentSource.create(url, xsb);
        this.newDocs.add(abstractWsdlSource);
        XMLStreamBufferResult r = new XMLStreamBufferResult(xsb);
        r.setSystemId((String)filename.value);
        return r;
    }
    
    public Result getSchemaOutput(String namespace, Holder<String> filename) {
        List<SDDocumentImpl> schemas = (List)this.nsMapping.get(namespace);
        if (schemas != null) {
            if (schemas.size() > 1) {
                throw new ServerRtException("server.rt.err", new Object[] { "More than one schema for the target namespace " + namespace });
            }
            filename.value = ((SDDocumentImpl)schemas.get(0)).getURL().toExternalForm();
            return null;
        }
        else {
            URL url = this.createURL((String)filename.value);
            MutableXMLStreamBuffer xsb = new MutableXMLStreamBuffer();
            xsb.setSystemId(url.toExternalForm());
            SDDocumentSource sd = SDDocumentSource.create(url, xsb);
            this.newDocs.add(sd);
            XMLStreamBufferResult r = new XMLStreamBufferResult(xsb);
            r.setSystemId((String)filename.value);
            return r;
        }
    }
    
    public SDDocumentImpl updateDocs() {
        for (SDDocumentSource doc : this.newDocs) {
            SDDocumentImpl docImpl = SDDocumentImpl.create(doc, this.serviceName, this.portTypeName);
            if (doc == this.concreteWsdlSource) {
                this.concreteWsdl = docImpl;
            }
            this.docs.add(docImpl);
        }
        return this.concreteWsdl;
    }
}
